/***************************************************************************
* Copyright (c) 2013 Abdurrahman AVCI <abdurrahmanavci@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the
* Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
***************************************************************************/

#include "AppsModel.h"

#include <QFile>
#include <QList>
#include <QTextStream>
#include <QStringList>
#include <QProcess>
#include <QPixmap>
#include <memory>
#include <pwd.h>
#include <QFileInfo>

#include <KWindowSystem/KWindowSystem>
#include <KWindowSystem/NETWM>
#include <QX11Info>
#include <QSettings>

namespace NSS {
    class Apps {
    public:
        QString name;
    };

    typedef std::shared_ptr<Apps> AppsPtr;

    class AppsModelPrivate {
    public:
        QList<AppsPtr> apps;
    };

    AppsIconProvider::AppsIconProvider() : QQuickImageProvider(QQuickImageProvider::Pixmap){
        for (auto wid: KWindowSystem::self()->windows()){
            KWindowInfo info(wid,0,NET::WM2WindowClass);
            QString name;
            if (info.valid()){
                name = info.windowClassName();
                if (name.compare("desktopWidget") == 0 || name.compare("nde-fileman") == 0 || name.compare("nde-panel") == 0 ) {
                    continue;
                }

                iconsList << KWindowSystem::self()->icon(wid);
            }
        }
    }

    AppsIconProvider::~AppsIconProvider(){

    }

    QPixmap AppsIconProvider::requestPixmap(const QString &id, QSize *size, const QSize &requestedSize){
        QPixmap pix;
        if (id.toInt() >= 0 && id.toInt() < iconsList.count()){
            pix = iconsList.at(id.toInt());
            pix.scaled(requestedSize.width() > 0 ? requestedSize.width() : 32, requestedSize.height() > 0 ? requestedSize.height() : 32);
        }
        return pix;
    }

    AppsModel::AppsModel(QObject *parent) : QAbstractListModel(parent), d(new AppsModelPrivate()) {

        QString name;

        for (auto wid: KWindowSystem::self()->windows())
        {
            KWindowInfo info(wid,0,NET::WM2WindowClass);
            if (info.valid())
            {
                name = info.windowClassName();
                if (name.compare("desktopWidget") == 0 || name.compare("nde-fileman") == 0 || name.compare("nde-panel") == 0 ) {
                    continue;
                }

                AppsPtr app { new Apps() };
                app->name = name;
                QFileInfo fi(QString("/usr/share/icons/Light/32x32/apps/"+name+".svg"));

                d->apps << app;
            }
        }

    }

    AppsModel::~AppsModel() {
        delete d;
    }

    QHash<int, QByteArray> AppsModel::roleNames() const {
        // set role names
        QHash<int, QByteArray> roleNames;
        roleNames[NameRole] = QByteArrayLiteral("name");

        return roleNames;
    }

    int AppsModel::rowCount(const QModelIndex &parent) const {
        return d->apps.length();
    }

    QVariant AppsModel::data(const QModelIndex &index, int role) const {
        if (index.row() < 0 || index.row() > d->apps.count())
            return QVariant();

        // get user
        AppsPtr app = d->apps[index.row()];

        // return correct value
        if (role == NameRole)
            return app->name;

        // return empty value
        return QVariant();
    }

    QString AppsModel::wallpaperPath() const
    {
        QString filePath = QString(qgetenv("HOME")) + "/.config/nde-fileman/nde/settings.conf";
        QFileInfo file(filePath);
        QString wallpaper = "/usr/share/backgrounds/default.png";
        if(file.exists() != false){
            QSettings settings(filePath, QSettings::IniFormat);
             wallpaper = settings.value("Desktop/Wallpaper").toString();
        }
        return  wallpaper;
    }

    int AppsModel::screensaverinterval() const
    {
        int  interval = 300;
        QSettings settings("nde","screensaver");
        if(settings.value("FadeoutTimer").toString() != nullptr)
        {
            interval = settings.value("FadeoutTimer").toInt();
        }
        return interval ;
    }
}
