/***************************************************************************
* Copyright (c) 2013 Abdurrahman AVCI <abdurrahmanavci@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the
* Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
***************************************************************************/

#ifndef NSS_APPSModel_H
#define NSS_APPSModel_H

#include <QAbstractListModel>
#include <KWindowSystem/KWindowSystem>
#include <QQuickImageProvider>
#include <QHash>
#include <QTimer>

namespace NSS {
    class AppsModelPrivate;

    class AppsIconProvider : public QQuickImageProvider{
    public:
        AppsIconProvider();
        ~AppsIconProvider();

        QPixmap requestPixmap(const QString &id, QSize *size, const QSize &requestedSize) override;

    private:
        QList<QPixmap> iconsList;
    };

    class AppsModel : public QAbstractListModel {
        Q_OBJECT
        Q_DISABLE_COPY(AppsModel)
        Q_PROPERTY(int count READ rowCount CONSTANT)
        Q_PROPERTY(QString wallpaperPath READ wallpaperPath CONSTANT)
        Q_PROPERTY(int screensaverinterval READ screensaverinterval CONSTANT)
    public:
        enum UserRoles {
            NameRole = Qt::UserRole + 1,
        };

        AppsModel(QObject *parent = 0);
        ~AppsModel();

        QHash<int, QByteArray> roleNames() const override;
        QString currentUser() const;

        int rowCount(const QModelIndex &parent = QModelIndex()) const override;
        QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
        QString wallpaperPath() const;  //锁屏背景
        int screensaverinterval() const; //锁屏到屏保的时间间隔
    private:
        AppsModelPrivate *d { nullptr };
    };
}

#endif // NSS_AppsModel_H
