/***************************************************************************
* Copyright (c) 2015 Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
* Copyright (c) 2013 Abdurrahman AVCI <abdurrahmanavci@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the
* Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
***************************************************************************/

#include "GreeterProxy.h"
#include "Messages.h"
#include "SocketWriter.h"
#include <X11/Xlib.h>
#undef Status
#include <QX11Info>
#include <QLocalSocket>
#include <QString>
#include <QDBusReply>
#include <QDBusInterface>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <QSettings>
#include <QFileInfo>
namespace NSS {
    class GreeterProxyPrivate {
    public:
//        SessionModel *sessionModel { nullptr };
        QLocalSocket *socket { nullptr };
        QString hostName;
        int tryCount { 0 };
        bool isVirtualMachine { false };
    };

    GreeterProxy::GreeterProxy(const QString &socket, QObject *parent) : QObject(parent), d(new GreeterProxyPrivate()) {
        d->socket = new QLocalSocket(this);
        // connect signals
        connect(d->socket, SIGNAL(connected()), this, SLOT(connected()));
        connect(d->socket, SIGNAL(disconnected()), this, SLOT(disconnected()));
        connect(d->socket, SIGNAL(readyRead()), this, SLOT(readyRead()));
        connect(d->socket, SIGNAL(error(QLocalSocket::LocalSocketError)), this, SLOT(error()));

        // connect to server
        d->socket->connectToServer(socket);
    }

    GreeterProxy::~GreeterProxy() {
        delete d;
    }

    const QString &GreeterProxy::hostName() const {
        return d->hostName;
    }

    bool GreeterProxy::isVirtualMachine() const {
        return d->isVirtualMachine;
    }

    bool GreeterProxy::isConnected() const {
        return d->socket->state() == QLocalSocket::ConnectedState;
    }

    void GreeterProxy::screensaveroff() {
        SocketWriter(d->socket) << quint32(GreeterMessages::ScreensaverOff) ;
    }

    void GreeterProxy::connected() {
        // log connection
        qDebug() << "Connected to the daemon.";

        // send connected message
        SocketWriter(d->socket) << quint32(GreeterMessages::Connect);
    }

    void GreeterProxy::disconnected() {
        // log disconnection
        qDebug() << "Disconnected from the daemon.";
    }

    void GreeterProxy::error() {
        qCritical() << "Socket error: " << d->socket->errorString();
    }

    bool GreeterProxy::checkVirtualMachine(){
        QProcess p;
        QStringList options;

        options << "-c" << "grep -q \"^flags.*\\ hypervisor\\ \" /proc/cpuinfo";
        p.start("/bin/bash", options);
        p.waitForFinished();
        p.waitForReadyRead();

        int exitCode = p.exitCode();
        return exitCode ? false : true;
    }

    QString GreeterProxy::wallpaperPath() const
    {
        QString filePath = QString(qgetenv("HOME")) + "/.config/nde-fileman/nde/settings.conf";
        QFileInfo file(filePath);
        QString wallpaper = "/usr/share/backgrounds/default.png";
        if(file.exists() != false){
            QSettings settings(filePath, QSettings::IniFormat);
             wallpaper = settings.value("Desktop/Wallpaper").toString();
        }
        return  wallpaper;
    }

    bool GreeterProxy::isChecked()const
    {
        QSettings settings("nde","screensaver");
        QString status = settings.value("Status").toString();
        if( status == "true")
        {
             return true;
        }
        return false;
    }

    QString GreeterProxy::screensaverPath() const
    {
        QString path = nullptr;
        if(isChecked()){
             path = wallpaperPath();
        }else {
            QSettings settings("nde","screensaver");
            path = settings.value("Screensaver").toString();
            QFileInfo file(path);
            if(file.exists() != true)
            {
                path = wallpaperPath();
            }
        }
        return path;
    }

    void GreeterProxy::readyRead() {
        // input stream
        QDataStream input(d->socket);

        while (input.device()->bytesAvailable()) {
            // read message
            quint32 message;
            input >> message;

            switch (DaemonMessages(message)) {
                case DaemonMessages::Capabilities: {
                    // log message
                    qDebug() << "Message received from daemon: Capabilities";

                    // read capabilities
                    quint32 capabilities;
                    input >> capabilities;
                    qDebug() << capabilities;
                    // parse capabilities

                    d->isVirtualMachine = checkVirtualMachine();
                }
                break;
                case DaemonMessages::HostName: {
                    // log message
                    qDebug() << "Message received from daemon: HostName";

                    // read host name
                    input >> d->hostName;

                    // emit signal
                    emit hostNameChanged(d->hostName);
                }
                break;
                default: {
                    // log message
                    qWarning() << "Unknown message received from daemon.";
                }
            }
        }
    }
}
