/***************************************************************************
* Copyright (c) 2013 Abdurrahman AVCI <abdurrahmanavci@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the
* Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
***************************************************************************/

#ifndef NSS_GREETERPROXY_H
#define NSS_GREETERPROXY_H

#include <QObject>
#include <QProcess>

class QLocalSocket;

namespace NSS {
    class GreeterProxyPrivate;
    class GreeterProxy : public QObject {
        Q_OBJECT
        Q_DISABLE_COPY(GreeterProxy)

        Q_PROPERTY(QString  hostName            READ hostName           NOTIFY hostNameChanged)
        Q_PROPERTY(bool     isVirtualMachine    READ isVirtualMachine   NOTIFY isVirtualMachineChanged)
        Q_PROPERTY(QString wallpaperPath READ wallpaperPath CONSTANT)
        Q_PROPERTY(QString screensaverPath READ screensaverPath CONSTANT)

    public:
        explicit GreeterProxy( const QString &socket, QObject *parent = 0);
        ~GreeterProxy();

        const QString &hostName() const;
        bool isConnected() const;
        bool isVirtualMachine() const;
        bool checkVirtualMachine();
        QString wallpaperPath() const;  //锁屏背景
        QString screensaverPath() const; //屏保背景
        bool  isChecked() const;
    public slots:
        void screensaveroff();
    private slots:
        void connected();
        void disconnected();
        void readyRead();
        void error();

    signals:
        void hostNameChanged(const QString &hostName);
        void isVirtualMachineChanged(bool isVirtualMachine);

    private:
        GreeterProxyPrivate *d { nullptr };
    };
}

#endif // NSS_GREETERPROXY_H
