#ifndef SCREENSAVERAPP_H
#define SCREENSAVERAPP_H

#include <QGuiApplication>
#include "vmdaemon_interface.h"
#include "Auth.h"
#define SaverApp ScreenSaverApp::instance()
class QLocalSocket;
class IdleWatcher;
class QProcess;
class ScreenSaver;
namespace NSS {
class PowerManager;
class SocketServer;
class LockScreenShortcut;
class LockScreenManager;
class SignalHandler;

class ScreenSaverApp : public QGuiApplication
{
    Q_OBJECT
public:
    explicit ScreenSaverApp(int &argc, char **argv);
    ~ScreenSaverApp();
    static ScreenSaverApp *instance() { return self; }

    PowerManager *powerManager() const;
    bool locked() const;
    void setIdleTime(int secs);
    void setIdleTime_t(int secs);
    static void setScreenSaver(QString) ;//设置屏保
    static void setInterval(int) ; // 设计时间间隔(min)
    static void setScreenStatus(QString);//设置是否锁定屏幕
    bool isLockscreen() const; //是否锁定屏幕，如果返回值true为锁屏
    static void setStatus(QString);//设置是否在屏保界面显示锁屏壁纸
signals:
    void stopped();
    void loggingIn(QLocalSocket *socket);
    void loginFailed(QLocalSocket *socket);
    void loginSucceeded(QLocalSocket *socket);

public slots:
    void slotRequestChanged();
    void slotAuthenticationFinished(const QString &user, bool success);
    void slotHelperFinished(Auth::HelperExitStatus status);
    void slotAuthInfo(const QString &message, Auth::Info info);
    void slotAuthError(const QString &message, Auth::Error error);
    void login(QLocalSocket *socket,
               const QString &user, const QString &password);
    void lockSession();
    void screensaverStartup();
    void handleSignal();
    void screensaveroff();
private:
    void unLock();
    static ScreenSaverApp *self;
    Auth *m_auth { nullptr };
    SocketServer *m_socketServer { nullptr };
    SocketServer *m_socketServer_t { nullptr };
    PowerManager *mPowerManager { nullptr };
    QLocalSocket *m_socket { nullptr };
    SignalHandler *m_signalHandler { nullptr };
    QString m_passPhrase;
    IdleWatcher *mIdleWatcher;
    QProcess *mLockGreeter;
    QProcess *mScreenGreeter;
    int mIdleTime;
    int mIdleTime_t;
    LockScreenShortcut *mShortcut;
    com::vm::daemon *daemonInterface;
    LockScreenManager *mLockScreenManager;
    ScreenSaver *mScreenSaver;
};
}
#endif // SCREENSAVERAPP_H
